/**************************************************************************
 * This file is part of the source of ApCoCoA Eclipse Plugin Project 
 * (Moccha).
 *
 *  Copyright (c) 2009 Stefan Schuster
 *
 * Visit http://apcocoa.org/ for more information regarding ApCoCoA
 * and ApCoCoALib.
 * Visit http://www.apcocoa.org/wiki/ApCoCoA:KnownIssues for bugs, problems 
 * and known issues.
 *
 * Moccha is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3 or later)
 * as published by the Free Software Foundation. A copy of the full
 * licence may be found in the file COPYING in this directory.
 *
 * Moccha is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Moccha; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 **************************************************************************/
package org.apcocoa.eclipse;

import java.io.IOException;

import org.apcocoa.eclipse.cocoawrapper.CoCoAProcessManager;
import org.apcocoa.eclipse.cocoawrapper.CoCoAServerManager;
import org.apcocoa.eclipse.console.ApCoCoAConsole;
import org.apcocoa.eclipse.core.ACPackageManager;
import org.apcocoa.eclipse.templates.ApCoCoAContextType;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.text.templates.ContextTypeRegistry;
import org.eclipse.jface.text.templates.persistence.TemplateStore;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.console.ConsolePlugin;
import org.eclipse.ui.console.IConsole;
import org.eclipse.ui.console.IConsoleManager;
import org.eclipse.ui.console.MessageConsole;
import org.eclipse.ui.editors.text.templates.ContributionContextTypeRegistry;
import org.eclipse.ui.editors.text.templates.ContributionTemplateStore;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

/**
 * The activator class controls the plug-in life cycle.
 * TODO it currently does much more, this has to be refactored.
 */
public class ApCoCoAPlugin extends AbstractUIPlugin {

	// The plug-in ID
	public static final String PLUGIN_ID = "org.apcocoa.eclipse";

	// Key to store custom templates. 
    private static final String CUSTOM_TEMPLATES_AC_KEY = "ACTemplates";

 // The context type registrys. 
    private static ContributionContextTypeRegistry gApCoCoAContextRegistry = null;
	
	// The template store
	private static TemplateStore gApCoCoATemplateStore;
	
	// Print debug messages on stdout?
	// TODO: Automate this for builds. If Release-Build then false, otherwise true.
	private static boolean DEBUG = true;
	
	// The shared instance
	/**
	 * @uml.property  name="plugin"
	 * @uml.associationEnd  
	 */
	private static ApCoCoAPlugin plugin;
	
	/**
	 * @uml.property  name="apCoCoAconsole"
	 * @uml.associationEnd  
	 */
	private ApCoCoAConsole apCoCoAconsole;
	
	
	
	/**
	 * @uml.property  name="fCoCoAProcessManager"
	 * @uml.associationEnd  
	 */
	private static CoCoAProcessManager fCoCoAProcessManager;
	
	private IExternalContribution fApCoCoAContribution;
	
	/**
	 * The constructor
	 */
	public ApCoCoAPlugin() {
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		ApCoCoAPlugin.debugLog("Start ApCoCoAPlugin");
		super.start(context);
		plugin = this;
		
		
		PlatformUI.getWorkbench().getPerspectiveRegistry()
				.setDefaultPerspective(IApCoCoAConstants.APCOCOA_PERSPECTIVE_ID);
		
		
		
		initContributionExtension();
		
		// TODO move this to the editor
		ACPackageManager.initializeACPackages(false);
		
		apCoCoAconsole = new ApCoCoAConsole("ApCoCoA");
		apCoCoAconsole.show(true);
		
		IPreferenceStore prefStore = getDefault().getPreferenceStore();
		boolean autostart = prefStore.getBoolean(IApCoCoAConstants.PREF_COCOA_AUTOSTART);
		if (autostart) {
			CoCoAProcessManager.getCoCoAProcessManager().restartApCoCoA();
		}
		
		checkConsoleWidthLimit();
		// CoCoAProcess.startCoCoA(console);
		// Location l1 = Platform.getInstallLocation();
		// printExtensionPointsInfo();
	}
	
	private void checkConsoleWidthLimit() {
		IPreferenceStore prefStore = getDefault().getPreferenceStore();
		boolean limitWidth = prefStore.getBoolean(IApCoCoAConstants.PREF_USE_CONSOLE_WIDTH);
		if (limitWidth) {
			boolean rsponse = MessageDialog.openQuestion(null, "Warning", 
					"The width of the ApCoCoA output shell is currently limited.\n" +
					"This feature can currently cause ApCoCoA to crash.\n" +
					"Do you want to disable this feature (recommended).");
			if (rsponse)
				prefStore.setValue(IApCoCoAConstants.PREF_USE_CONSOLE_WIDTH, false);
		}
	}
	
	private void initContributionExtension() {
		// System.out.println("Init Contr Ext start...");
		IExtensionRegistry reg = Platform.getExtensionRegistry();
		// Note: it could in the future be possible to have more than one
		// (ap)cocoa installations. For now, however, we just take the first one
		// we find.
		fApCoCoAContribution = null;
		IConfigurationElement[] extensions = reg
				.getConfigurationElementsFor("org.apcocoa.external.contribution");
		if (extensions.length == 0)
			return;
		
		IConfigurationElement element = extensions[0];
		try {
			fApCoCoAContribution = (IExternalContribution) element.createExecutableExtension("class");
		} catch (CoreException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		System.out.println("Init Contr Ext done." + ((fApCoCoAContribution == null) ? "null" : "NN"));
	}

//	private void printExtensionPointsInfo() {
//		StringBuffer buffer = new StringBuffer();
//		IExtensionRegistry reg = Platform.getExtensionRegistry();
//		IConfigurationElement[] extensions = reg
//				.getConfigurationElementsFor("org.apcocoa.external.contribution");
//		System.out.println("Got " + extensions.length + " extensions.");
//		for (int i = 0; i < extensions.length; i++) {
//			IConfigurationElement element = extensions[i];
//			buffer.append("Name: " + element.getDeclaringExtension().getLabel());
//			buffer.append("\n");
//			buffer.append(element.getAttribute("cocoa_text_path"));
//			buffer.append("\n");
//			String cost = "unknown";
//			String info = "";
//			if (element.getAttribute("wordlist_path") != null) {
//				cost = element.getAttribute("wordlist_path");
//			}
//			try {
//				IExternalContribution topping = (IExternalContribution) element.createExecutableExtension("class");
//				info = String.valueOf(topping.getPluginLocation());
//			} catch (CoreException e) {
//				info = e.toString(); // obviously for demo purposes only
//			}
//
//			buffer.append(cost);
//			buffer.append("\n");
//			buffer.append(info);
//			buffer.append("\n");
//		}
//		System.out.println("++++ Extension points info: ++++");
//		System.out.println(buffer.toString());
//	}
	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		super.stop(context);
		try {
			plugin = null;
			ApCoCoAPlugin.debugLog("[ApCoCoAPlugin.stop() called]");
			
			// Stop all processes that still might be running.
			CoCoAServerManager.stopCoCoAServer();
			CoCoAProcessManager.getCoCoAProcessManager().stopApCoCoA();
		} catch (Exception e) {
			System.out.println(e.getMessage());
		}
	}

	/**
	 * Returns the shared instance
	 *
	 * @return the shared instance
	 */
	public static ApCoCoAPlugin getDefault() {
		return plugin;
	}

	/**
	 * @return
	 * @uml.property  name="plugin"
	 */
	public static ApCoCoAPlugin getPlugin() {
		return getDefault();
	}
	
	public ApCoCoAConsole getConsole() {
		return apCoCoAconsole;
	}
	
	public MessageConsole getServerConsole() {
		return findConsole("ApCoCoA-Server");
	}
	
	/**
	 * Returns the standard display to be used. The method first checks, if
	 * the thread calling this method has an associated display. If so, this
	 * display is returned. Otherwise the method returns the default display.
	 */
	public static Display getStandardDisplay() {
		Display display= Display.getCurrent();
		if (display == null) {
			display= Display.getDefault();
		}
		return display;		
	}
	
	/**
	 * Returns an image descriptor for the image file at the given
	 * plug-in relative path
	 *
	 * @param path the path
	 * @return the image descriptor
	 */
	public static ImageDescriptor getImageDescriptor(String path) {
		return imageDescriptorFromPlugin(PLUGIN_ID, path);
	}
	
	/** */
	public static CoCoAProcessManager getApCoCoAProcessManager() {
		return fCoCoAProcessManager;
	}
	
	private MessageConsole findConsole(String name) {
	      ConsolePlugin plugin = ConsolePlugin.getDefault();
	      IConsoleManager conMan = plugin.getConsoleManager();
	      IConsole[] existing = conMan.getConsoles();
	      for (int i = 0; i < existing.length; i++)
	         if (name.equals(existing[i].getName()))
	            return (MessageConsole) existing[i];
	      //no console found, so create a new one
	      MessageConsole myConsole = new MessageConsole(name, null);
	      conMan.addConsoles(new IConsole[]{myConsole});
	      return myConsole;
	   }

	public static TemplateStore getApCoCoATemplateStore() {
		if (gApCoCoATemplateStore == null) {
			gApCoCoATemplateStore = new ContributionTemplateStore(getApCoCoAContextTypeRegistry(),
                    plugin.getPreferenceStore(),
                    CUSTOM_TEMPLATES_AC_KEY);
            try {
            	gApCoCoATemplateStore.load();
            } catch (IOException e) {
                //e.printStackTrace();
                ApCoCoAPlugin.log("Loading TeX template store", e);
                throw new RuntimeException(e);
            }
        }
        return gApCoCoATemplateStore;
	}

	public static ContextTypeRegistry getApCoCoAContextTypeRegistry() {
		   
	    
	        if (gApCoCoAContextRegistry == null) {
	            // create an configure the contexts available in the template editor
	        	gApCoCoAContextRegistry = new ContributionContextTypeRegistry();
	        	gApCoCoAContextRegistry.addContextType(ApCoCoAContextType.AC_CONTEXT_TYPE);
	        }
	        return gApCoCoAContextRegistry;
	   
	}

	   /**
     * Create ae error-level status object out of textual message.
     * These status-objects are needed when creating new CoreExceptions.  
     * Used by e.g. the project creation wizard.
     * @param msg error message to display in error log
     * @param t exception
     * @return new error-level status message 
     */
    public static IStatus stat(String msg, Throwable t) {
        return new Status(IStatus.ERROR, getPluginId(), IStatus.ERROR, msg, t);
    }
    
    /**
     * This is equivalent to calling <code>stat(msg, null)</code>.
     * @param msg error message to display in error log
     * @return new error-level status message 
     */
    public static IStatus stat(String msg) {
        return stat(msg, null);
    }
    
    /**
     * Display a message in the Eclipse's Error Log.
     * This is equivalent to calling <code>log(msg, t, IStatus.ERROR)</code>.
     * @param msg error message to display in error log
     * @param t exception
     */
    public static void log(String msg, Throwable t) {
        log(msg, t, IStatus.ERROR);
    }
    
    /**
     * Display a message in the Eclipse's Error Log.
     * Used by e.g. the project creation wizard.
     * @param msg error message
     * @param t exception
     * @param level one of the error levels defined in the <code>IStatus</code> -interface
     */
    public static void log(String msg, Throwable t, int level) {
        IStatus stat = new Status(level, getPluginId(), level, msg, t);
        getDefault().getLog().log(stat);
    }
    
    public static void debugLog(String s) {
    	if (DEBUG) {
    		System.out.println(s);
    	}
    }
    
    /**
     * Returns the name of the plugin.
     * 
     * Used by project creation wizard.
     * @return unique id of this plugin
     */
    public static String getPluginId() {
        return getDefault().getBundle().getSymbolicName();
    }
    
    public IExternalContribution getExternalContribution() {
    	return fApCoCoAContribution;
    }
    
}
