/**************************************************************************
 * This file is part of the source of ApCoCoA Eclipse Plugin Project 
 * (Moccha).
 *
 *  Copyright (c) 2009-2010 Stefan Schuster
 *
 * Visit http://apcocoa.org/ for more information regarding ApCoCoA
 * and ApCoCoALib.
 * Visit http://www.apcocoa.org/wiki/ApCoCoA:KnownIssues for bugs, problems 
 * and known issues.
 *
 * Moccha is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3 or later)
 * as published by the Free Software Foundation. A copy of the full
 * licence may be found in the file COPYING in this directory.
 *
 * Moccha is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Moccha; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 **************************************************************************/
package org.apcocoa.eclipse.cocoawrapper;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Map;

import org.apcocoa.eclipse.ApCoCoAPlugin;
import org.apcocoa.eclipse.IApCoCoAConstants;
import org.apcocoa.eclipse.console.ApCoCoAConsole;
import org.eclipse.cdt.utils.spawner.Spawner;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.debug.core.DebugException;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationType;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.debug.core.IStreamListener;
import org.eclipse.debug.core.Launch;
import org.eclipse.debug.core.model.IProcess;
import org.eclipse.debug.core.model.IStreamMonitor;
import org.eclipse.debug.core.model.IStreamsProxy;
import org.eclipse.debug.core.model.RuntimeProcess;
import org.eclipse.debug.ui.IDebugUIConstants;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

import zzunused.CAResponseManager;
import zzunused.OutputConnector;
import zzunused.OutputConsoleConnector;


// TODO: Refactor: Rename to CoCoAProcessManager or CoCoAProcessFactory
/**
 * @author  schuster
 */
public class CoCoAProcess {

	public static final String APCOCOA_LAUNCH_CONFIGURATION_TYPE = "org.apcocoa.eclipse.launchConfigurationType";

	private final static String ATTR_CAPTURE_IN_CONSOLE = IDebugUIConstants.ATTR_CAPTURE_IN_CONSOLE;
	private final static String ATTR_CAPTURE_OUTPUT = DebugPlugin.ATTR_CAPTURE_OUTPUT;
	private final static String ATTR_PROCESS_FACTORY_ID = DebugPlugin.ATTR_PROCESS_FACTORY_ID;
	
	// This is the actual external process that is managed by this class.
	private static Process mProcess;
	 
	/** Singleton pattern: reference to the only instance of this class.
	 * @uml.property  name="fCoCoAProcess"
	 * @uml.associationEnd  
	 */
	private static CoCoAProcess fCoCoAProcess = null;
	
	// private static InputStream mErrorStream;
	// InputStream mInputStream;
	// private static OutputStream mOutputStream;
	// private static BufferedReader mIn;
	// private static BufferedWriter mOut;
	
	/**
	 * @uml.property  name="oc"
	 * @uml.associationEnd  
	 */
	private static OutputConnector oc = null;
	
	/**
	 * @uml.property  name="occ"
	 * @uml.associationEnd  
	 */
	private static OutputConsoleConnector occ = null;
	
	/**
	 * @uml.property  name="fConsole"
	 * @uml.associationEnd  
	 */
	private static ApCoCoAConsole fConsole = null;
	
	private static IProcess ip;
	
	private static ILaunch launch;
	
	private static IStreamListener outputListener;
	private static IStreamListener errorListener;
	private static IStreamMonitor outputMonitor;
	private static IStreamMonitor errorMonitor;
	
	private CoCoAProcess() {
		
	}
	
	private CoCoAProcess(String cocoa) {
		// fCoCoAProcess = null;
	}
	
	/**
	 * 
	 * @param cocoa Must be the absolute path to and including the cocoa executable. 
	 * @throws IOException 
	 */
	private CoCoAProcess(String cocoa, InputStream in, OutputStream out, OutputStream err) throws IOException {
		// mOutputStream = mProcess.getOutputStream();
		// mOut = new BufferedWriter(new OutputStreamWriter(mOutputStream));
		// new ConsoleDisplayMgr("CoCoA");
	}
	
	protected static CoCoAProcess getCoCoAProcess() {
		if (fCoCoAProcess == null) {
			fCoCoAProcess = new CoCoAProcess();
			createExternalProcess2();
			// connectStreams(null, null);
		}
		return fCoCoAProcess;
	}
	
	/**
	 * Much of this code is taken from 
	 */
	private static void createExternalProcess2() {

		
		
		ILaunchConfigurationWorkingCopy workingCopy = null;
		ILaunchConfiguration configuration = null;
		String configurationName = null;
		try {

			configurationName = "New_configuration";
			// configuration = CoCoAProcess.findLaunchConfiguration(configurationName);
			configuration = CoCoAProcess.createNewLaunchConfiguration();
		}
		catch (CoreException exception) {

			Shell shell = Display.getDefault().getActiveShell();
			String title = "LocalTerminalMessages.errorTitleCouldNotConnectToTerminal";
			String text = "LocalTerminalMessages.errorLaunchConfigurationNoLongerExists";
			String message = NLS.bind(text, configurationName);
			IStatus status = new Status(IStatus.ERROR, ApCoCoAPlugin.PLUGIN_ID, message);
			ErrorDialog.openError(shell, title, null, status);
			return;
		}
		try {

			String oldFactoryID = configuration.getAttribute(ATTR_PROCESS_FACTORY_ID, (String)null);
			workingCopy = configuration.getWorkingCopy();
			workingCopy.setAttribute(ATTR_CAPTURE_OUTPUT, true);
			workingCopy.setAttribute(ATTR_CAPTURE_IN_CONSOLE, true);
			workingCopy.setAttribute(ATTR_PROCESS_FACTORY_ID, CoCoAProcessFactory.ID);
			configuration = workingCopy.doSave();
			try {

				launch = configuration.launch(ILaunchManager.RUN_MODE, null);
			}
			finally {

				// The process factory ID is used to distinguish between launches that originate
				// from the terminal connector and launches that originate from the launch dialog.
				// After launching, the original ID is restored so that the launch is not mistaken
				// as originating from the terminal connector UI when it is launched via the launch
				// dialog the next time:
				//
				workingCopy = configuration.getWorkingCopy();
				workingCopy.setAttribute(ATTR_PROCESS_FACTORY_ID, oldFactoryID);
				workingCopy.doSave();
			}

			// To prevent a console from being allocated, the launch will actually not contain a
			// reference to the runtime process. The process has to be obtained from the
			// LocalTerminalProcessRegistry instead:
			//
			
			IProcess[] tProcesses = launch.getProcesses();
			if (tProcesses == null || tProcesses.length == 0) {
				System.err.println("oh oh");
				return;
			}
			System.out.println("We have " + tProcesses.length + " processes.");
			ip = tProcesses[0];
			
			IStreamsProxy streamsProxy = ip.getStreamsProxy();

			// Hook up standard input:
			//
			// terminalToLocalProcessStream = new LocalTerminalOutputStream(process, settings);

			// Hook up standard output:
			//
			outputMonitor = streamsProxy.getOutputStreamMonitor();
			// outputListener = ApCoCoAPlugin.getPlugin().getConsole();
			outputListener = new CoCoAStreamAppendedListener(ApCoCoAPlugin.getPlugin().getConsole());
//			try {
//				Thread.sleep(500);
//			} catch (InterruptedException e) {
//				// TODO Auto-generated catch block
//				e.printStackTrace();
//			}
			outputMonitor.addListener(outputListener);
			
			String r = outputMonitor.getContents();
			System.out.println(">>" + r + "<<");
			outputListener.streamAppended(r, outputMonitor);

			// Hook up standard error:
			//
//			errorMonitor = streamsProxy.getErrorStreamMonitor();
//			errorListener = new LocalTerminalOutputListener(control, settings);
//			errorMonitor.addListener(errorListener);
//			errorListener.streamAppended(errorMonitor.getContents(), errorMonitor);
			//
			// TODO: add proper synchronization for incoming data from stdout and stderr:
			//       currently, the data gets sometimes processed in the wrong order, for example,
			//       the next prompt (which shells like bash print to stderr) sometimes appears
			//       before the command's proper output that was sent to stdout. For example,
			//       you get:
			//
			//       $ echo hello
			//       $ hello
			//
			//       instead of the correct output of:
			//
			//       $ echo hello
			//       hello
			//       $

			// Listen for process termination and update the terminal state:
			//
			// TODO DebugPlugin.getDefault().addDebugEventListener(this);
		}
		catch (CoreException exception) {
			// System.out.println("ohoh: " + exception.toString());

			Shell shell = Display.getDefault().getActiveShell().getShell();
			ErrorDialog.openError(shell, "CoCoAProcess", "ApCoCoA could not be started.", exception.getStatus());
//			Logger.logException(exception);
		}
	}
	
//	private static void createExternalProcess() {
//		ApCoCoAPlugin.debugLog("[CoCoAProcess.createExternalProcess]");
//		String cocoaPath = ApCoCoAPlugin.getDefault().getPreferenceStore().getString(IApCoCoAConstants.PREF_AC_EXEC);
//		
//		ApCoCoAPlugin.debugLog("Starting cocoa: " + cocoaPath);
//		
//		// ProcessBuilder pb = new ProcessBuilder(cocoaPath);
//		File workingDir = (new File(cocoaPath)).getParentFile().getAbsoluteFile(); 
//		// pb.directory(cocoaParent);
//		
//		// mErrorStream = mProcess.getErrorStream();
//		// mInputStream = mProcess.getInputStream();
//		// mIn = new BufferedReader(new InputStreamReader(mInputStream));
//		try {
//			String[] cmd = {cocoaPath};
//			// String[] cmd = {"/usr/local/bin/apcocoa"};
//			
//			// TODO: This does not work on windows!!
//			// If you provide environment variables, cocoa_text cannot
//			// open the socket to the server.
//			// mProcess = DebugPlugin.exec(cmd, cocoaParent, CoCoAServerManager.getEnvironmentVars());
//			mProcess = DebugPlugin.exec(cmd, workingDir);
//			
//			// mProcess = pb.start();
//			Launch launch = new Launch(null, ILaunchManager.RUN_MODE, null);
//			
//			ip = DebugPlugin.newProcess(launch, mProcess, "ApCoCoA Process");
//			
//			ApCoCoAPlugin.debugLog("[*****] Done");
//			// launch.
//			IStreamsProxy sp = ip.getStreamsProxy();
//			final IStreamMonitor ism = sp.getOutputStreamMonitor();
//			// if (ism instanceof IFlushableStreamMonitor)
//			// 	((IFlushableStreamMonitor) ism).setBuffered(true);
//			// ApCoCoAPlugin.getPlugin().getConsole().
//			// ApCoCoAPlugin.debugLog(ism.getContents());
//			//////////////////////////////////////////////////////
//			// This line probably caused the deadlock in the restart event:
//			// ((ApCoCoAConsole)ApCoCoAPlugin.getPlugin().getConsole()).sendResponse(ism.getContents());
//			//////////////////////////////////////////////////////
//			
//			// Here, the splash screen is printed.
//			// TODO must be refactored.
//			ism.addListener(ApCoCoAPlugin.getPlugin().getConsole());
//			ApCoCoAPlugin.getPlugin().getConsole().sendResponse(ism.getContents());
//			// ism.addListener(CAResponseManager.getInstance());
//			// We have to call this asynchronously to avoid deadlocks.
//			ApCoCoAPlugin.getStandardDisplay().asyncExec(new Runnable() {
//				public void run() {
//					// TODO: Assure that clearing ALWAYS comes before appending.
//					// System.out.println("async...");
//					// ApCoCoAPlugin.getPlugin().getConsole().sendResponse(ism.getContents());
//					// .clearAndAppend(ism);
//					
//				}
//			});
//			
//			ApCoCoAPlugin.debugLog("[+++++] Done");
//		//} catch (IOException e) {
//		//	e.printStackTrace();
//			
//		} catch (CoreException e) {
//			e.printStackTrace();
//			ApCoCoAPlugin.getStandardDisplay().asyncExec(new Runnable() {
//				public void run() {
//					ApCoCoAConsole c = ApCoCoAPlugin.getPlugin().getConsole();
//					
//					c.appendError("\nERROR: ApCoCoA could not be started!\n"
//							+ "Please recheck the settins under\n" 
//							+ "\tWindow > Preferences... > ApCoCoA Preferences");
//				}
//			});
//			
//			// e.printStackTrace();
//		}
//		
//	}
	
	protected void connectOutputStream(OutputStream out) {	
	}
	
	protected void connectInputStream() {	
	}
	
	protected static void startCoCoA(ApCoCoAConsole console) {
		fConsole = console;
		getCoCoAProcess();
	}
	
	private static void detachStreams() {
		ApCoCoAPlugin.debugLog("[CoCoAProcess.detachStreams]");
		// TODO: Cleaning up: e.g. detach connections between console and process.
		if (oc != null) {
			// synchronized (oc) {
				oc.detach();
				oc = null;
			// }
		}
		ApCoCoAPlugin.debugLog("[CoCoAProcess.detachStreams] ret..");
	}
	
//	protected static void stopCoCoA() {
//		ApCoCoAPlugin.debugLog("[CoCoAProcess.stopCoCoA]");
//		
//		detachStreams();
//		
//		// ApCoCoAPlugin.debugLog(mProcess.toString());
//		
//		// if (mProcess != null) {
//		try {
//			
//			mProcess.destroy();
//			ApCoCoAPlugin.debugLog("[CoCoAProcess.stopCoCoA] exitValue:" + mProcess.exitValue());
//			mProcess = null;
//		} catch (Exception e) {
//			ApCoCoAPlugin.debugLog("[CoCoAProcess.stopCoCoA] error " + e.getMessage());
//		}
//		// }
//		ApCoCoAPlugin.debugLog("[CoCoAProcess.stopCoCoA] ret..");
//	}
	
	protected static void stopCoCoA2() {
		
		ApCoCoAPlugin.debugLog("Stopping CoCoA (2)  - Begin");
		// XXX ApCoCoAPlugin.getPlugin().getConsole().clearConsole();
		try {
			final IStreamMonitor tIsm = ip.getStreamsProxy().getOutputStreamMonitor();
			if (tIsm != null)
				ApCoCoAPlugin.debugLog("CoCoAProcess.stopCoCoA2(): " + tIsm.toString());
			else 
				ApCoCoAPlugin.debugLog("tIsm was null");
			ApCoCoAPlugin.debugLog("Getting Console.");
			ApCoCoAPlugin acplugin = ApCoCoAPlugin.getPlugin();
			if (acplugin == null)
				ApCoCoAPlugin.debugLog("Plugin is null!");
			if (acplugin != null) {
				final ApCoCoAConsole tConsole = ApCoCoAPlugin.getPlugin().getConsole();
			
				ApCoCoAPlugin.debugLog("CoCoAProcess.stopCoCoA2():  Removing listeners.");
				new Thread(new Runnable() {
					public void run() {
						// tIsm.removeListener(tConsole);
						tIsm.removeListener(outputListener);
					}
				}).start(); 
			
				ApCoCoAPlugin.debugLog("CoCoAProcess.stopCoCoA2():  Listeners removed.");
			}
			ip.terminate();
			ApCoCoAPlugin.debugLog("CoCoAProcess.stopCoCoA2():  Terminated.");
			
			
			fCoCoAProcess = null;
		} catch (DebugException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		ApCoCoAPlugin.debugLog("Stopping CoCoA (2)  - End");
	}
	
	protected static void restartCoCoA(ApCoCoAConsole console) {
		
		stopCoCoA2();
		
		if (ip.isTerminated())
			ApCoCoAPlugin.debugLog("Terminated!");
		else 
			ApCoCoAPlugin.debugLog("NOT Terminated!");
		//synchronized (ApCoCoAPlugin.getPlugin().getWorkbench()) {
			// ApCoCoAConsole c = ApCoCoAPlugin.getDefault().getConsole();
			// c.clearConsole();
		createExternalProcess2();
		// }
		// connectStreams(null, null);		
//		if (console == null && fConsole != null) 
//			getCoCoAProcess();
//		else if (console == null && fConsole == null)
//			// TODO: Throw error
//			;
//		else {
//			console.clearConsole();
//			startCoCoA(console);
//		}
	}
	
	protected static void restartCoCoA() {
		restartCoCoA(null);
	}
	
	protected OutputStream getOutputStream() {
		return mProcess.getOutputStream();
	}
	
	protected InputStream getInputStream() {
		return mProcess.getInputStream();
	}
	
	protected void sendString(String s) throws IOException {
		sendString(s, false);
	}
	
	protected void sendString(String s, boolean showOnConsole) throws IOException {
		/* TODO:
		 * 1) Synchronized?
		 * 2) Start another Thread?
		 * 3) Status-Info on the current state of this class. e.g. isProcessRunning() */
		ApCoCoAPlugin.debugLog("CoCoAProcess.sendString(s,b): Called.");
		String r;
		StringBuffer sb = new StringBuffer();
		
		String tLineSeperator = System.getProperty("line.separator");

		// ApCoCoAPlugin.debugLog(s);
		
		// ApCoCoAPlugin.debugLog(">>" + s);
		// ConsoleDisplayMgr.getDefault().println(s, ConsoleDisplayMgr.MSG_USER_INPUT);
		if (showOnConsole)
			ApCoCoAPlugin.getDefault().getConsole().appendCommands(s + "\n\n");
		//else
		//	ApCoCoAPlugin.getDefault().getConsole().sendCommands("[...]");
		
		// Write to CoCoA process.
		// TODO: \r\n is not quite platform independent...
		// ApCoCoAPlugin.debugLog(s);
		ApCoCoAPlugin.debugLog("CoCoAProcess.sendString(s,b): Writing to stream proxy...");
		ip.getStreamsProxy().write("\n" + s);
		ApCoCoAPlugin.debugLog("CoCoAProcess.sendString(s,b): ... done.");
		// mOut.write(s + "\n");
		// mOut.flush();
		// ApCoCoAPlugin.debugLog(">> ww");
		/* while ((r = mIn.readLine()) != null && !r.startsWith("-----")) {
			ApCoCoAPlugin.debugLog(r);
			sb.append(r);
			sb.append("\n");
			ApCoCoAPlugin.debugLog("-");
		}
		ApCoCoAPlugin.debugLog("--"); */
		ApCoCoAPlugin.debugLog("CoCoAProcess.sendString(s,b): Leaving.");
		
	}

	public void interrupt() {
		if (ip != null && ip instanceof ExtendedProcess) {
			((ExtendedProcess) ip).interrupt();
				
		}
	}
	
	/**
	 * 
	 * @param out where the InputStream (the response) of the process is written to.
	 * @param in where we get the input for the process from.
	 */
	/* private static void connectStreams(OutputStream out, InputStream in) {
		ApCoCoAPlugin.debugLog("[CoCoAProcess.connectStreams]");
		if (out == null) {
			MessageConsoleStream mcs;
			
			if (fConsole != null)
				mcs = fConsole.getMessageStream();
			else {
				ApCoCoAPlugin.debugLog("[CoCoAProcess.connectStreams] No console present, using plugin console.");
				mcs = ApCoCoAPlugin.getDefault().getConsole().getMessageStream();
			}
				
			occ = new OutputConsoleConnector("CoCoA-OC", 
					mProcess.getInputStream(), 
					ApCoCoAPlugin.getDefault().getConsole());
					/* ConsoleDisplayMgr.getDefault().
						getNewMessageConsoleStream(ConsoleDisplayMgr.MSG_COCOA_RESPONSE)); */
		/*	occ.start();
		} else {
			oc = new OutputConnector("CoCoA-OC", 
					mProcess.getInputStream(), 
					out);
			oc.start();
		}
		
		if (in == null) {
			mOutputStream = mProcess.getOutputStream();
			mOut = new BufferedWriter(new OutputStreamWriter(mOutputStream));
			try {
				mOut.flush();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		} else {
			OutputConnector ic = new OutputConnector("CoCoA-IC",
					in,
					mProcess.getOutputStream());
			ic.start();
		}
	} */
	
	// public void run() {
	
	// }
	
	/* protected int close() throws IOException {
		this.sendString("Ciao;");
		return mProcess.exitValue();
	} */
	
	private static ILaunchConfiguration createNewLaunchConfiguration() throws CoreException {
		ILaunchConfiguration configuration;
		ILaunchManager manager = DebugPlugin.getDefault().getLaunchManager();
		ILaunchConfigurationType tType = manager.getLaunchConfigurationType(APCOCOA_LAUNCH_CONFIGURATION_TYPE);
		configuration = tType.newInstance(null, "CoCoA Launch Configuration");
		return configuration;
	}
	
	
	/**
	 * Finds a launch configuration by its name.
	 *
	 * @param name the name of the launch configuration
	 * @return the corresponding {@link ILaunchConfiguration} object or <code>null</code> if the
	 * configuration could not be found
	 * @throws CoreException if there was a general problem accessing launch configurations
	 */
	private static ILaunchConfiguration findLaunchConfiguration(String name) throws CoreException {

		ILaunchConfiguration[] configuration;
		ILaunchManager manager = DebugPlugin.getDefault().getLaunchManager();
		ILaunchConfigurationType tLaunchConfigType = manager.getLaunchConfigurationType(APCOCOA_LAUNCH_CONFIGURATION_TYPE);
		configuration = manager.getLaunchConfigurations(tLaunchConfigType);
		// configuration = new ILaunchConfiguration[] {createNewLaunchConfiguration()};
		int numberOfConfigurations = configuration.length;
		System.out.println("Hey, we have " + numberOfConfigurations + " configurations!");
		for (int index = 0; index < numberOfConfigurations; index++) {
			System.out.println(configuration[index].getName());
			if (configuration[index].getName().equals(name)) {

				return configuration[index];
			}
		}
		// String error = NLS.bind(LocalTerminalMessages.noSuchLaunchConfiguration, name);
		String error = "blah";
		throw new CoreException(new Status(IStatus.ERROR, ApCoCoAPlugin.PLUGIN_ID, error));
	}
}
