/**************************************************************************
 * This file is part of the source of ApCoCoA Eclipse Plugin Project 
 * (Moccha).
 *
 *  Copyright (c) 2009 Stefan Schuster
 *
 * Visit http://apcocoa.org/ for more information regarding ApCoCoA
 * and ApCoCoALib.
 * Visit http://www.apcocoa.org/wiki/ApCoCoA:KnownIssues for bugs, problems 
 * and known issues.
 *
 * Moccha is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3 or later)
 * as published by the Free Software Foundation. A copy of the full
 * licence may be found in the file COPYING in this directory.
 *
 * Moccha is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Moccha; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 **************************************************************************/
package org.apcocoa.eclipse.cocoawrapper;

import java.io.File;
import java.util.List;
import java.util.Vector;

import org.apcocoa.eclipse.ApCoCoAPlugin;
import org.apcocoa.eclipse.IApCoCoAConstants;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.DebugException;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.debug.core.Launch;
import org.eclipse.debug.core.model.IProcess;
import org.eclipse.debug.internal.core.LaunchManager;
import org.eclipse.debug.internal.ui.viewers.update.LaunchManagerProxy;
import org.eclipse.debug.internal.ui.views.console.ProcessConsole;
import org.eclipse.debug.ui.console.ConsoleColorProvider;
import org.eclipse.debug.ui.console.IConsoleColorProvider;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.console.ConsolePlugin;
import org.eclipse.ui.console.IConsole;
import org.eclipse.ui.console.IConsoleManager;
import org.eclipse.ui.console.IConsoleView;
import org.eclipse.ui.console.MessageConsole;
import org.eclipse.ui.internal.console.ConsoleManager;

public class CoCoAServerManager {

	public static void startApCoCoAServer() {
		ApCoCoAPlugin.debugLog("[CoCoAServerManager.startApCoCoAServer()]");
		try {
			// Array containing the command to call and its arguments.
			String[] cmdLst = getCocoaServerCmdList();
			File parent = new File(cmdLst[0]).getParentFile();
			
			// System.out.println(cmdLst);
			
			Process mProcess = DebugPlugin.exec(cmdLst, parent);
			// mProcess = pb.start();
			Launch launch = new Launch(null, ILaunchManager.RUN_MODE, null);
			
			IProcess p = DebugPlugin.newProcess(launch, mProcess, IApCoCoAConstants.SRV_CONSOLE_NAME);
			IConsoleColorProvider ccp = new ConsoleColorProvider();
			
			ProcessConsole prc = new ProcessConsole(p, ccp);
			
			IConsole[] cs = {prc};
			ConsolePlugin.getDefault().getConsoleManager().addConsoles(cs);
			ConsolePlugin.getDefault().getConsoleManager().showConsoleView(prc);
			
			// Finally, remove terminated consoles from the drop down list of available consoles.
			IConsole[] consoles = ConsolePlugin.getDefault().getConsoleManager().getConsoles();
			List<IConsole> l = new Vector<IConsole>();
			for (int i=0; i<consoles.length; i++) {
				String consoleName = consoles[i].getName();
				// System.out.println(consoleName);
				if (consoleName.equals("<terminated> ApCoCoA Server")) {
					l.add(consoles[i]);
				}
			}
			IConsole[] csToRemove = {};
			ConsolePlugin.getDefault().getConsoleManager().removeConsoles(l.toArray(csToRemove));
			
		} catch (CoreException e) {
			ApCoCoAPlugin.debugLog(e.getMessage());
		}
		
	}
	
	private static String[] getCocoaServerCmdList() {
		IPreferenceStore prefStore = ApCoCoAPlugin.getDefault().getPreferenceStore();
		String cocoaServerPath = prefStore.getString(IApCoCoAConstants.PREF_AC_SERVER_EXEC);
		String cocoaServerPort = prefStore.getString(IApCoCoAConstants.PREF_AC_SERVER_PORT).trim();

		String[] cmdLst;
		if (cocoaServerPort == null || cocoaServerPort.equals("")) {
			cmdLst = new String[1];
			cmdLst[0] = cocoaServerPath;
		} else {
			cmdLst = new String[3];
			cmdLst[0] = cocoaServerPath;
			cmdLst[1] = "-p";
			cmdLst[2] = cocoaServerPort;
		}
		return cmdLst;
	}
	
	public static String[] getEnvironmentVars() {
		String[] envVars = new String[2];
		IPreferenceStore prefStore = ApCoCoAPlugin.getDefault().getPreferenceStore();
		String cocoaPckagesPath = prefStore.getString(IApCoCoAConstants.PREF_AC_PACKAGES_PATH).trim();
		String cocoaRcFilePath = prefStore.getString(IApCoCoAConstants.PREF_AC_COCOARC_PATH).trim();
		
		// COCOA_PACKAGES
		envVars[0] = "COCOA_PACKAGES=" 
			+ (cocoaPckagesPath == null ? "" : cocoaPckagesPath);
		
		// COCOARC
		envVars[1] = "COCOARC=" + cocoaRcFilePath;
		
//		System.out.println(envVars[0]);
//		System.out.println(envVars[1]);
		
		return envVars;
	}
	
	public static void stopCoCoAServer() {
		IConsoleManager conman = ConsolePlugin.getDefault().getConsoleManager();
		IConsole[] consoles = conman.getConsoles();
		for (int i=0; i<consoles.length; i++) {
			IConsole tmpConsole = consoles[i];
			if (tmpConsole.getName().equals(IApCoCoAConstants.SRV_CONSOLE_NAME) && tmpConsole instanceof ProcessConsole) {
				IProcess process = ((ProcessConsole)tmpConsole).getProcess();
				try {
					process.terminate();
				} catch (DebugException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	}
}
