package org.apcocoa.eclipse.cocoawrapper;

import java.nio.CharBuffer;
import java.text.CharacterIterator;
import java.text.StringCharacterIterator;

import org.apcocoa.eclipse.console.ApCoCoAConsole;
import org.apcocoa.eclipse.console.ICoCoAConsole;
import org.eclipse.debug.core.IStreamListener;
import org.eclipse.debug.core.model.IStreamMonitor;

public class CoCoAStreamAppendedListener implements IStreamListener {

	private static final String NEW_LINE = "\n";

	private static final char NEW_LINE_CHAR = '\n';

	private static final int NUL = -1;

	private static final int E = 0;

	private static final int ER = 1;

	private static final int ERR = 2;

	private static final int ERRO = 3;

	private static final int ERROR = 4;

	private static boolean fFound = false;
	
	private StringBuilder fBuilder;

	private ICoCoAConsole fConsole;

	private int fState = -1;

	public CoCoAStreamAppendedListener(ICoCoAConsole console) {
		fConsole = console;
		fBuilder = new StringBuilder();
	}

	public void streamAppended(String text, IStreamMonitor monitor) {
		System.out.println("CoCoAStreamAppendList: " + text);
		if (text == null || text.equals(""))
			return;

		StringCharacterIterator iter = new StringCharacterIterator(text);
		for (char c = iter.first(); c != CharacterIterator.DONE; c = iter
				.next()) {
			processChar(c);
		}
		handleRest();
	}

	private void processChar(char c) {
		fBuilder.append(c);
		switch (c) {
		case NEW_LINE_CHAR:
			handleCompletedLine();
			break;
		case 'E':
			handle_E();
			break;
		case 'R':
			handle_R();
			break;
		case 'O':
			handle_O();
			break;
		default:
			break;
		}
	}

	private void handle_O() {
		if (fFound)
			return;
		if (fState == ERR)
			fState++;
		else
			fState = NUL;

	}

	private void handle_R() {
		if (fFound)
			return;
		if (fState == E || fState == ER || fState == ERRO)
			fState++;
		else
			fState = NUL;
		if (fState == ERROR)
			fFound = true;
	}

	private void handle_E() {
		if (fFound)
			return;
		if (fState == NUL)
			fState++;
		else
			fState = NUL;
	}

	private void handleRest() {
		if (!possibleRest()) {
			fConsole.appendResponse(fBuilder.toString());
			fBuilder = new StringBuilder();
			fState = NUL;
			fFound = false;
		}
	}

	private boolean possibleRest() {
		return fState > NUL;
	}

	private void handleCompletedLine() {
		int index = fBuilder.indexOf("ERROR");
		if (index >= 0) {
			fConsole.appendResponse(fBuilder.substring(0, index));
			fConsole.appendError(modifyErrorLine(fBuilder.substring(index)));
		} else {
			fConsole.appendResponse(fBuilder.toString());
		}
		fState = NUL;
		fFound = false;
		fBuilder = new StringBuilder();
	}
	
	private String modifyErrorLine(String line) {
		String lastDisplayName = CoCoAProcessManager.getCoCoAProcessManager().getLastDisplayName();
		String lastTmpFileName = CoCoAProcessManager.getCoCoAProcessManager().getLastTmpFileName();
		return line.replace(lastTmpFileName, lastDisplayName);
	}
}
