/**************************************************************************
 * This file is part of the source of ApCoCoA Eclipse Plugin Project 
 * (Moccha).
 *
 *  Copyright (c) 2009 Stefan Schuster
 *
 * Visit http://apcocoa.org/ for more information regarding ApCoCoA
 * and ApCoCoALib.
 * Visit http://www.apcocoa.org/wiki/ApCoCoA:KnownIssues for bugs, problems 
 * and known issues.
 *
 * Moccha is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3 or later)
 * as published by the Free Software Foundation. A copy of the full
 * licence may be found in the file COPYING in this directory.
 *
 * Moccha is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Moccha; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 **************************************************************************/
package org.apcocoa.eclipse.console;

import org.apcocoa.eclipse.ApCoCoAPlugin;
import org.eclipse.ui.console.ConsolePlugin;
import org.eclipse.ui.console.IConsole;
import org.eclipse.ui.console.IConsoleFactory;
import org.eclipse.ui.console.IConsoleManager;

public abstract class AbstractConsoleFactory implements IConsoleFactory {
	
	public abstract IConsole getConsole();
	
	public void openConsole() {
		showConsole(getConsole());
	}
	
	public static void showConsole(IConsole console) {
		// IConsole console = this.getConsole();
		if (console != null) {
			IConsoleManager manager = ConsolePlugin.getDefault().getConsoleManager();
			IConsole[] existing = manager.getConsoles();
			boolean exists = false;
			for (int i = 0; i < existing.length; i++) {
				if(console == existing[i])
					exists = true;
			}
			if(! exists)
				manager.addConsoles(new IConsole[] {console});
			manager.showConsoleView(console);
		}
	}
	
	public static void closeConsole() {
		IConsoleManager manager = ConsolePlugin.getDefault().getConsoleManager();
		ApCoCoAConsole console = ApCoCoAPlugin.getPlugin().getConsole();
		if (console != null) {
			manager.removeConsoles(new IConsole[] {console});
			ConsolePlugin.getDefault().getConsoleManager().addConsoleListener(console.new MyLifecycle());
		}
	}
	
}
