package org.apcocoa.eclipse.core;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apcocoa.eclipse.ApCoCoAPlugin;
import org.apcocoa.eclipse.IApCoCoAConstants;
import org.apcocoa.eclipse.editors.RecursiveFileTraverser;
import org.eclipse.jface.text.templates.Template;

public class ACPackageManager {

	private static String[] wordList = {};
	
	private static boolean isInitialized = false;
	
	private static Map<String, String> aliases;
	
	private static List<CAFunDef> funDefs = new Vector<CAFunDef>();
	
	private static class PackageFileTraverser extends RecursiveFileTraverser {

		public PackageFileTraverser(File parentDir) {
			super(parentDir);
		}
		
		public PackageFileTraverser(File parentDir, FilenameFilter filter) {
			super(parentDir, filter);
		}

		@Override
		protected void handleFile(File file) {
			ApCoCoAPlugin.debugLog("Handling File " + file.getAbsolutePath());
			try {
				BufferedReader br = new BufferedReader(new FileReader(file));
				StringBuffer buffer = new StringBuffer();
				String line;
				// String newline = System.getProperty("line.separator");
				while ((line = br.readLine()) != null)
					buffer.append(line).append(" ");
				List<String> pkgs = getPackageDefinitions(buffer);
				ApCoCoAPlugin.debugLog("Number of Package definitions: " + pkgs.size());
				// for (String s: pkgs)
				// 	ApCoCoAPlugin.debugLog("> " + s);
				if (pkgs.size()>1) {
					ApCoCoAPlugin.debugLog("File containes more than 2 package defs, there will be no prefixes! "
					 		+ file.getAbsolutePath());
					// pack
				}
				String packageDef = (pkgs.size() == 1) ? pkgs.get(0) : "";
				// ApCoCoAPlugin.debugLog("*** Package: " + packageDef);
				List<CharSequence> myfundefs = getFunctionDefinitions(buffer);
				ApCoCoAPlugin.debugLog("Number of found function definitions: " + myfundefs.size());
				// for (CharSequence s : myfundefs)
					// ApCoCoAPlugin.debugLog("> " + s);
				for (CharSequence s : myfundefs) {
					funDefs.add(getCAFunDef(s, packageDef, file.getName()));
					// ApCoCoAPlugin.debugLog(getCAFunDef(s, packageDef).getTemplatePattern());
				}
				
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
		/**
		 * Extracts all CoCoAL function definitions from a given CharSequence.
		 * @param cs
		 * @return A list where every element contains a string in the form of  
		 * "Define <i>name</i> ([<i>args, args,...</i>]).
		 */
		private List<CharSequence> getFunctionDefinitions(final CharSequence cs) {
			List<CharSequence> result = new Vector<CharSequence>();
			Pattern p = Pattern.compile("(Define\\s+\\w+\\s*\\([\\s\\w,]*\\))");
			Matcher m = p.matcher(cs);
			while (m.find())
				result.add(cs.subSequence(m.start(), m.end()));
			return result;
		}
		
		private List<String> getPackageDefinitions(final CharSequence cs) {
			List<String> result = new ArrayList<String>();
			Pattern p = Pattern.compile("Package\\s+[\\$|/\\w]+\\s+");
			Matcher m = p.matcher(cs);
			if (!m.find())
				return result;
			
			String s = cs.subSequence(m.start(), m.end()).toString();
			s = s.replaceAll("Package", "").trim();
			// ApCoCoAPlugin.debugLog("Got package definition for " + s);
			result.add(s);
			return result;
		}
		
		/**
		 * Extract the function name and possible arguments from a CoCoAL function 
		 * definition.
		 * @param cs A CharSequence containing a string in the form of "Define <i>name</i> ([<i>args, args,...</i>]).
		 * @param packageDefinition the Package definition
		 * @return The corresponding CAFunDef object
		 */
		private CAFunDef getCAFunDef(CharSequence cs, String packageDefinition, String fileName) {
			Pattern p = Pattern.compile("(Define)|([\\w]+)");
			Matcher m = p.matcher(cs);
			
			// "Define"
			if (!m.find()) return null; 
			
			// The name of the function
			if (!m.find()) return null; 
			
			CAFunDef fd = new CAFunDef(cs.subSequence(m.start(), m.end()).toString());
			// System.out.println("Function " + fd.name);
			
			// Possible arguments.
			// ApCoCoAPlugin.debugLog("Function: " + fd.name);
			List<String> l = new Vector<String>();
			String tmp = "";
			while(m.find()) {
				String s = cs.subSequence(m.start(), m.end()).toString();
				// System.out.println(s);
				if (s.trim().equals("Var")) {
					tmp = "Var_";
				} else {
					l.add(tmp + s);
					tmp = "";
				}
				// ApCoCoAPlugin.debugLog("  " + cs.subSequence(m.start(), m.end()).toString());
			}
			fd.arguments = l;
			
			// Package definitions
			if (packageDefinition != null)
				fd.packageName = packageDefinition.replaceAll("\\$coclib", "");
			else
				fd.packageName = packageDefinition;
			
			if (packageDefinition != null)
				fd.alias = aliases.get(packageDefinition);
			
			fd.file = (fileName == null) ? "" : fileName;
			
			return fd;
		}
	}
	
	private static class CAFunDef implements Comparable<CAFunDef>{
		
		public String name;
		
		public List<String> arguments;
		
		private String packageName;
		
		private String alias;
		
		private String file;
		
		private String documentation;
		
		public String toString() {
			return alias + ":" + packageName + ":" + name + ":" + arguments.size();
		}
		
		public CAFunDef(String name) {
			this.name = name;
			this.file = "";
			this.arguments = new Vector<String>();
		}

		public int compareTo(CAFunDef arg0) {
			return getQualifiedNameForComparison().compareTo(arg0.getQualifiedNameForComparison());
		}
		
		public String getTemplateName() {
			return getQualifiedName();
		}
		
		public String getDescription() {
			return getQualifiedName() + " (" + this.file + ")";
		}
		
		public String getTemplatePattern() {
			StringBuffer sb = new StringBuffer();
			sb.append(getQualifiedName()).append("(");
			int i = 0;
			for (String s: arguments) {
				if (i++ > 0)
					sb.append(", ");
				sb.append("${").append(s).append("}");
			}
			sb.append(")${cursor}");
			return sb.toString();
		}
		
		public String getQualifiedName() {
			if (alias != null && !alias.equals(""))
				return alias + "." + name;
			if (packageName != null && !packageName.equals(""))
				return packageName + "." + name;
			return name;
		}
		
		private String getQualifiedNameForComparison() {
			if (alias != null)
				return alias + "." + name;
			if (packageName != null)
				return packageName + "." + name;
			return "." + name;
		}
	}
	
	/**
	 * Currently also initializes the wordlist
	 * @param force
	 */
	public static void initializeACPackages(boolean force) {
		if (!force && isInitialized)
			return;
		
		String tPackageDirString = ApCoCoAPlugin.getDefault().getPreferenceStore()
			.getString(IApCoCoAConstants.PREF_AC_PACKAGES_PATH);
		File tPackageDir = new File (tPackageDirString);
		initializeAliases(tPackageDir);
		
		if (funDefs == null)
			funDefs = new Vector<CAFunDef>();
		funDefs.clear();
		
		FilenameFilter f = new FilenameFilter() {
			public boolean accept(File dir, String name) {
				return name.endsWith("cpkg");
			}
			
		};
		PackageFileTraverser trav = new PackageFileTraverser(tPackageDir, f);
		trav.traverse(); // As a side effect, fills funDefs.
		
		// (2) Initialize the Wordlist.
		String tWordlistFileString = ApCoCoAPlugin.getDefault().getPreferenceStore()
			.getString(IApCoCoAConstants.PREF_AC_WORDLIST_FILE_PATH);
		File tWordlistFile = new File(tWordlistFileString);
		initializeWordlist(tWordlistFile);
	}
	
	/**
	 * We assume, that aliases.coc is inside the packageDir and is the
	 * only file to consider for global alias definitions.
	 * 
	 * Moreover, we assume, that the "Alias .... ;" statement is the only statement
	 * in the file where assignments ":=" occur.
	 * 
	 * This simplification will be removed as soon as we have an own parser/interpreter.
	 * 
	 * @param packageDir
	 */
	private static void initializeAliases(File packageDir) {
		if (aliases == null)
			aliases = new HashMap<String, String>();
		aliases.clear();
		File alias = new File(packageDir, "aliases.coc");
		if (!alias.exists()) {
			ApCoCoAPlugin.log("Cannot find file aliases.coc", new FileNotFoundException());
			return;
		}
		try {
			// ApCoCoAPlugin.debugLog("Reading file " + alias.getAbsolutePath());
			BufferedReader br = new BufferedReader(new FileReader(alias));
			StringBuffer buffer = new StringBuffer();
			String line;
			String newline = System.getProperty("line.separator");
			while ((line = br.readLine()) != null)
				buffer.append(line).append(newline);
			Pattern p = Pattern.compile("(\\s*\\w+\\s*:=\\s*[\\w+|\\$|/]+\\s*)");
			Matcher m = p.matcher(buffer);
			while (m.find()) {
				extractAndAddAliasPair(buffer.subSequence(m.start(), m.end()));
			}
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}
	
	private static void extractAndAddAliasPair(CharSequence cs) {
		// ApCoCoAPlugin.debugLog("extractAndAddAliasPair called");
		Pattern p = Pattern.compile("([\\w]+|:=|[\\w|/|\\$]+)");
		Matcher m = p.matcher(cs);
		if (!m.find())
			return;
		String alias = cs.subSequence(m.start(), m.end()).toString();
		if (!m.find() || !cs.subSequence(m.start(), m.end()).toString().equals(":="))
			return;
		if (!m.find())
			return;
		String aliased = cs.subSequence(m.start(), m.end()).toString();
		// ApCoCoAPlugin.debugLog("Adding alias-pair <" + aliased + ", " + alias + ">");
		aliases.put(aliased, alias);
	}
	
	private static void initializeWordlist(File wordlistFile) {
		if (isInitialized)
			return;
		Vector<String> tmpVec = new Vector<String>();
		try {
			BufferedReader reader = new BufferedReader(new FileReader(wordlistFile));
			String s;
			while ((s = reader.readLine()) != null)
				tmpVec.add(s);
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		isInitialized = true;
		wordList = tmpVec.toArray(wordList);
	}
	
	public static Template[] getFunctionCompletionTemplates() {
		
		Collections.sort(funDefs);
		
		if (funDefs == null || funDefs.size() == 0)
			return new Template[] {};
		
		Template[] ts = new Template[funDefs.size()];
		
		int i=0;
		for (CAFunDef fd : funDefs) {
			// TODO !!! CONTEXT TYPE """ XXX
			Template t = new Template(
					fd.getQualifiedName(), 
					fd.getDescription(), 
					"de", 
					fd.getTemplatePattern(),
					false);
			ts[i++] = t;
		}
			
		return ts;
	}
	
//	private static String getFunctionTemplateString(CAFunDef fd) {
//		StringBuffer sb = new StringBuffer();
//		sb.append(fd.name).append("(");
//		int i = 0;
//		for (String s: fd.arguments) {
//			if (i++ > 0)
//				sb.append(", ");
//			sb.append("${").append(s).append("}");
//		}
//		sb.append(")${cursor}");
//		return sb.toString();
//	}
	
	public static String[] getCompletionProposals() {
		// return new String[] {"A", "B", "C"};
		return wordList;
	}

}
