package org.apcocoa.eclipse.core;

import org.apcocoa.eclipse.ApCoCoAPlugin;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.ITextSelection;
import org.eclipse.jface.text.TextSelection;
import org.eclipse.jface.text.TextUtilities;


/**
 * This class extracts CoCoALanguage specific "local" information from selections.
 * Much is stolen from pydev...
 * @author shellcomputer
 *
 */
public class ApCoCoASelection {

	private IDocument doc;
    private ITextSelection textSelection;

	
	
	public ApCoCoASelection(IDocument document, int offset) {
		this.doc = document;
        this.textSelection = new TextSelection(doc, offset, 0);
	}

    public static String getIndentationFromLine(String selection) {
        int firstCharPosition = getFirstCharPosition(selection);
        return selection.substring(0, firstCharPosition);
    }

    public String getIndentationFromLine() {
        return getIndentationFromLine(getCursorLineContents());
    }
    
    public String getCursorLineContents() {
        try {
            int start = getStartLine().getOffset();
            int end = getEndLine().getOffset() + getEndLine().getLength();
            return this.doc.get(start, end-start);
        } catch (BadLocationException e) {
        	ApCoCoAPlugin.log("", e);
        }
        return "";
    }
    
    /**
     * @param src
     * @return
     */
    public static int getFirstCharPosition(String src) {
        int i = 0;
        boolean breaked = false;
        while (i < src.length()) {
            if (Character.isWhitespace(src.charAt(i)) == false && src.charAt(i) != '\t') {
                i++;
                breaked = true;
                break;
            }
            i++;
        }
        if (!breaked){
            i++;
        }
        return (i - 1);
    }

    /**
     * @return Returns the startLine.
     */
    public IRegion getStartLine() {
        try {
            return getDoc().getLineInformation(getStartLineIndex());
        } catch (BadLocationException e) {
            // Log.log(e);
        	ApCoCoAPlugin.log("", e);
        }
        return null;
    }

    /**
     * @return Returns the endLine.
     */
    public IRegion getEndLine() {
        try {
            int endLineIndex = getEndLineIndex();
            if(endLineIndex == -1){
                return null;
            }
            return getDoc().getLineInformation(endLineIndex);
        } catch (BadLocationException e) {
        	ApCoCoAPlugin.log("", e);
        }
        return null;
    }
	


    /**
     * @return Returns the startLineIndex.
     */
    public int getStartLineIndex() {
        return this.getTextSelection().getStartLine();
    }


    /**
     * @return Returns the endLineIndex.
     */
    public int getEndLineIndex() {
        return this.getTextSelection().getEndLine();
    }
    
    /**
     * @return Returns the doc.
     */
    public IDocument getDoc() {
        return doc;
    }
    

    /**
     * @return Returns the textSelection.
     */
    public ITextSelection getTextSelection() {
        return textSelection;
    }
    
    /**
     * @return the delimiter that should be used for the passed document
     */
    public static String getDelimiter(IDocument doc){
        return TextUtilities.getDefaultLineDelimiter(doc);
    }

}
