package org.apcocoa.eclipse.core;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.CharBuffer;

import org.apcocoa.eclipse.ApCoCoAPlugin;
import org.apcocoa.eclipse.IApCoCoAConstants;

/**
 * Manages the cocoa configuration file set in the preference store.
 * 
 * @author Stefan Schuster
 *
 */
public class RcFileManager {

	private static final String LINE_SEP = System.getProperty("line.separator");
	
	private static final String BEGIN_CONFIG_MARK = 
			"-- BEGIN OF AUTOMATICALLY GENERATED CONTENT" +
			LINE_SEP +
			"-- DO NOT EDIT!" +
			LINE_SEP;
	
	private static final String END_CONFIG_MARK = 
			"-- END OF AUTOMATICALLY GENERATED CONTENT" +
			LINE_SEP;
	
	
	
	public static void initRcFile() {
		// System.out.println("initRcFile called.");
		// rc file as defined in user preferences.
		String rcPath = ApCoCoAPlugin.getDefault().getPreferenceStore()
							.getString(IApCoCoAConstants.PREF_AC_COCOARC_PATH);
		File rcFile = new File (rcPath);
		StringBuffer sb = new StringBuffer();
		// Read file content
		if (rcFile.exists() && rcFile.canRead()) {
			try {
				BufferedReader r = new BufferedReader(new FileReader(rcFile));
				String currLine;
				while ((currLine = r.readLine()) != null)
					sb.append(currLine).append(LINE_SEP);
				r.close();
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
		// Build the new automatically generated part.
		StringBuffer newContent = new StringBuffer();
		newContent.append(BEGIN_CONFIG_MARK);
		String port = ApCoCoAPlugin.getDefault().getPreferenceStore()
						.getString(IApCoCoAConstants.PREF_AC_SERVER_PORT).trim();
		if (port != null && ! port.equals("")) {
			newContent.append("MEMORY.CoCoAServerPort := \"");
			newContent.append(port);
			newContent.append("\";\n");
		}
		newContent.append(END_CONFIG_MARK);
		
		// Adapt already existing content.
		if (sb.indexOf(BEGIN_CONFIG_MARK) >= 0) {
//			System.out.println("> " + sb.indexOf(BEGIN_CONFIG_MARK));
//			System.out.println("> " + sb.indexOf(END_CONFIG_MARK));
//			System.out.println("> " + END_CONFIG_MARK.length());
			sb.replace(sb.indexOf(BEGIN_CONFIG_MARK), 
					sb.indexOf(END_CONFIG_MARK) + END_CONFIG_MARK.length(),
					newContent.toString());
		} else {
			if (sb.length() > 0)
				sb.append(LINE_SEP);
			sb.append(newContent);
		}
		
		// Write file
		if (rcFile.exists() && rcFile.canWrite()) {
			try {
				BufferedWriter bw = new BufferedWriter(new FileWriter(rcFile));
				bw.write(sb.toString());
				bw.flush();
				bw.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
}
