package org.apcocoa.eclipse.core;

import java.util.ArrayList;
import java.util.List;

import org.apcocoa.eclipse.core.cache.Cache;
import org.apcocoa.eclipse.core.cache.LRUCache;

public class Utils {
    /**
     * Splits the given string in a list where each element is a line.
     * 
     * @param string string to be splitted.
     * @return list of strings where each string is a line.
     * 
     * @note the new line characters are also added to the returned string.
     */
    public static List<String> splitInLines(String string) {
        ArrayList<String> ret = new ArrayList<String>();
        int len = string.length();

        char c;
        FastStringBuffer buf = new FastStringBuffer();

        for (int i = 0; i < len; i++) {
            c = string.charAt(i);

            buf.append(c);

            if (c == '\r') {
                if (i < len - 1 && string.charAt(i + 1) == '\n') {
                    i++;
                    buf.append('\n');
                }
                ret.add(buf.toString());
                buf.clear();
            }
            if (c == '\n') {
                ret.add(buf.toString());
                buf.clear();

            }
        }
        if (buf.length() != 0) {
            ret.add(buf.toString());
        }
        return ret;

    }
    
    
    public static String replaceAll(String string, String replace, String with) {
        FastStringBuffer ret = new FastStringBuffer();
        int len = string.length();
        int replaceLen = replace.length();
        
        for (int i = 0; i < len; i++) {
            if(i+replaceLen > len){
                ret.append(string.charAt(i));
                continue;
            }
            String s = string.substring(i, i+replaceLen);
            if(s.equals(replace)){
                ret.append(with);
                i = i+replaceLen-1;
            }else{
                ret.append(s.charAt(0));
            }
        }

        return ret.toString();
    }
    
    /**
     * Creates a string of spaces of the designated length.
     * @param width number of spaces you want to create a string of
     * @return the created string
     */
    public static String createSpaceString(int width) {
        String existing = widthToSpaceString.getObj(width);
        if(existing != null){
            return existing;
        }
        StringBuffer b = new StringBuffer(width);
        while (width-- > 0){
            b.append(' ');
        }
        String newStr = b.toString();
        widthToSpaceString.add(width, newStr);
        return newStr;
    }
    
    /**
     * Small cache to hold strings only with spaces (so that each width has a created string).
     */
    private static Cache<Integer, String> widthToSpaceString = new LRUCache<Integer, String>(8);
   
}
