package org.apcocoa.eclipse.editors;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Vector;

import org.apcocoa.eclipse.ApCoCoAPlugin;
import org.apcocoa.eclipse.core.ACPackageManager;
import org.apcocoa.eclipse.templates.ApCoCoAContextType;
import org.apcocoa.eclipse.templates.ApCoCoATemplateCompletionProcessor;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.ITextViewer;
import org.eclipse.jface.text.contentassist.CompletionProposal;
import org.eclipse.jface.text.contentassist.ContentAssistEvent;
import org.eclipse.jface.text.contentassist.ContextInformation;
import org.eclipse.jface.text.contentassist.ContextInformationValidator;
import org.eclipse.jface.text.contentassist.ICompletionListener;
import org.eclipse.jface.text.contentassist.ICompletionProposal;
import org.eclipse.jface.text.contentassist.IContentAssistProcessor;
import org.eclipse.jface.text.contentassist.IContextInformation;
import org.eclipse.jface.text.contentassist.IContextInformationValidator;
import org.eclipse.jface.text.templates.GlobalTemplateVariables;
import org.eclipse.jface.text.templates.Template;
import org.eclipse.jface.text.templates.TemplateCompletionProcessor;
import org.eclipse.jface.text.templates.TemplateContextType;
import org.eclipse.swt.graphics.Image;

public class ApCoCoAConentAssistProcessor implements IContentAssistProcessor {
	
	protected static int state = 0;
	
	protected final static int SHOW_ALL = 0;
	
	protected final static int SHOW_ONLY_TEMPLATES = 1;
	
	protected final static int SHOW_ONLY_KEYWORDS = 2;

    protected int whatToShow = SHOW_ALL;

    protected ApCoCoAContentAssistant contentAssistant = null;
	
	private ApCoCoATemplateCompletionProcessor templateCompletionProc = new ApCoCoATemplateCompletionProcessor();
	

    /**
     * This is the content assistant that is used to start this processor.
     */
    // protected PyContentAssistant pyContentAssistant;

    //-------- cycling through regular completions and templates
   
	public ApCoCoAConentAssistProcessor(ApCoCoAContentAssistant assi) {
		this.contentAssistant = assi;
		
		contentAssistant.addCompletionListener(new ICompletionListener(){

            public void assistSessionEnded(ContentAssistEvent event) {
            }

            public void assistSessionStarted(ContentAssistEvent event) {
                startCycle();
            }

            public void selectionChanged(ICompletionProposal proposal, boolean smartToggle) {
                //ignore
            }
		});
		
	}
    
    public void startCycle(){
        whatToShow = SHOW_ALL;
    }
    
    protected void doCycle() {
        if(whatToShow == SHOW_ALL){
            whatToShow = SHOW_ONLY_TEMPLATES;
        } else if (whatToShow == SHOW_ONLY_TEMPLATES) {
			whatToShow = SHOW_ONLY_KEYWORDS;
		} else{
            whatToShow = SHOW_ALL;
        }
    }
    
    /**
     * Updates the status message.
     */
    public void updateStatus(){
        if(whatToShow == SHOW_ALL){
            // contentAssistant.setIterationStatusMessage("Press %s for templates.");
        	contentAssistant.setStatusMessage("Press Ctrl+Space for templates.");
        } else if (whatToShow == SHOW_ONLY_TEMPLATES) {
			
        	contentAssistant.setStatusMessage("Press Ctrl+Space for keyword completions.");
		}
        else{
        	// contentAssistant.setIterationStatusMessage("Press %s for default completions.");
        	contentAssistant.setStatusMessage("Press Ctrl+Space for default completions.");
        }
    }
	
	
	public ICompletionProposal[] computeCompletionProposals(ITextViewer viewer,
			int offset) {
		
		updateStatus();
		
		// TODO: Check, if some text is selected -- if so, just return.
		ApCoCoAPlugin.debugLog("computeCompletionProposal called.");
		String prefix = computeActivationString(viewer, offset);
		
		ArrayList<ICompletionProposal> proposals = new ArrayList<ICompletionProposal>();
		
		// 1) Compute ApCoCoA function proposals
		if (whatToShow == SHOW_ALL)
			computeApCoCoAProposals(viewer, offset, proposals);
		
		// 2) Compute Template proposals
		if (whatToShow == SHOW_ONLY_TEMPLATES || whatToShow == SHOW_ALL)
			computeAndAddTemplateProposals(viewer, offset, proposals, prefix);
		
		if (whatToShow == SHOW_ONLY_KEYWORDS || whatToShow == SHOW_ALL)
			computeAndAddKeywordProposals(viewer, offset, proposals, prefix);
			
		ICompletionProposal[] result = new ICompletionProposal[0];
		result = proposals.toArray(result);
		
		ApCoCoAPlugin.debugLog("Number of proposals: " + result.length);
		
		doCycle();
		
		return result;
	}

	public IContextInformation[] computeContextInformation(ITextViewer viewer,
			int offset) {
		String now = new Date(System.currentTimeMillis()).toString();
		IContextInformation[] proposals = new IContextInformation[] { new ContextInformation(
				null, now) };
		return proposals;
	}

	public char[] getCompletionProposalAutoActivationCharacters() {
		return new char[] {};
	}

	public char[] getContextInformationAutoActivationCharacters() {
		return new char[] {};
	}

	public IContextInformationValidator getContextInformationValidator() {
		return new ContextInformationValidator(this);
	}

	public String getErrorMessage() {
		return Messages.getString("ApCoCoAConentAssistProcessor.1"); //$NON-NLS-1$
	}
	
	private String computeActivationString(ITextViewer viewer, int offset) {
		IDocument doc = viewer.getDocument();
		int start = offset;
		String result = "";
		try {
			while (start > 0) {
				if (Character.isWhitespace(doc.getChar(start-1))) break;
				--start;
			}
			result = doc.get(start, offset-start);
		} catch (BadLocationException e) {
		}
		return result;
	}

	private void computeAndAddTemplateProposals(ITextViewer viewer, int offset, List<ICompletionProposal> proposals, String prefix) {
		ArrayList<ICompletionProposal> tProps = new ArrayList<ICompletionProposal>();
		this.templateCompletionProc.addTemplateProposals(viewer, offset, tProps, prefix);
		// TODO: Sorting etc.
		proposals.addAll(tProps);
	}
	
	private void computeAndAddKeywordProposals(ITextViewer viewer, int offset,
			ArrayList<ICompletionProposal> proposals, String prefix) {
		
		String[] fgProposals = ACPackageManager.getCompletionProposals();
		for (int i = 0; i < fgProposals.length; i++) {
			if (fgProposals[i].toLowerCase().startsWith(prefix.toLowerCase())) {
				IContextInformation info = new ContextInformation(
						fgProposals[i],
						MessageFormat.format(
							Messages.getString("CompletionProcessor.Proposal.ContextInfo.pattern"), 
							new Object[] { fgProposals[i] })); //$NON-NLS-1$
				proposals.add( new CompletionProposal(
						fgProposals[i],
						offset - prefix.length(),
						prefix.length(),
						fgProposals[i].length(),
						null,
						fgProposals[i],
						info,
						MessageFormat.format(
								Messages.getString(
										"CompletionProcessor.Proposal.hoverinfo.pattern"), 
										new Object[] { fgProposals[i] })) //$NON-NLS-1$
				);
			}
		}
		
	}

	private void computeApCoCoAProposals(ITextViewer viewer, int offset, List<ICompletionProposal> proposals) {
		IContentAssistProcessor processor = new TemplateCompletionProcessor() {
			protected Template[] getTemplates(String contextTypeId) {
				return ACPackageManager.getFunctionCompletionTemplates();
			}
			protected TemplateContextType getContextType(ITextViewer viewer, IRegion region) {
				TemplateContextType type = new TemplateContextType("de", "Sample");
				type.addResolver(new GlobalTemplateVariables.Cursor());
				// type.addResolver(new GlobalTemplateVariables.User());
				return type;
				// return ApCoCoAPlugin.getApCoCoAContextTypeRegistry().getContextType(ApCoCoAContextType.AC_CONTEXT_TYPE);
			}
			protected Image getImage(Template template) {
				return null;
			}
			
			protected String extractPrefix(ITextViewer viewer, int offset) {
				ApCoCoAPlugin.debugLog("Offset: " + offset);
				IDocument doc = viewer.getDocument();
				int start = offset;
				String result = "";
				try {
					while (start > 0) {
						if (Character.isWhitespace(doc.getChar(start-1))) break;
						--start;
					}
					result = doc.get(start, offset-start);
				} catch (BadLocationException e) {
				}
				ApCoCoAPlugin.debugLog("Result: " + result);
				return result;
			}
		};
		ICompletionProposal[] tProps = processor.computeCompletionProposals(viewer, offset);
		for (int i=0; i<tProps.length; i++) {
			// We have to review the result again since it gives too much results otherwise.
			String ds = tProps[i].getDisplayString().toLowerCase().trim();
			if (ds.startsWith(computeActivationString(viewer, offset).toLowerCase())
					&& ! ds.startsWith("man."))
				proposals.add(tProps[i]);
		}
	}
	
//	private void oldProposalComputation(String prefix, int offset) {
//		
//		String[] fgProposals = WordlistManager.getCompletionProposals();
//		Vector<ICompletionProposal> tmpVec = new Vector<ICompletionProposal>();
//		for (int i = 0; i < fgProposals.length; i++) {
//			if (fgProposals[i].toLowerCase().startsWith(prefix.toLowerCase())) {
//			IContextInformation info = new ContextInformation(
//					fgProposals[i],
//					MessageFormat
//							.format(
//									Messages
//											.getString("CompletionProcessor.Proposal.ContextInfo.pattern"), new Object[] { fgProposals[i] })); //$NON-NLS-1$
//			tmpVec.add( new CompletionProposal(
//					fgProposals[i],
//					offset - prefix.length(),
//					prefix.length(),
//					fgProposals[i].length(),
//					null,
//					fgProposals[i],
//					info,
//					MessageFormat
//							.format(
//									Messages
//											.getString("CompletionProcessor.Proposal.hoverinfo.pattern"), new Object[] { fgProposals[i] })) //$NON-NLS-1$
//			);
//			}
//		}
//	}
}
