package org.apcocoa.eclipse.editors;

import org.eclipse.jface.bindings.TriggerSequence;
import org.eclipse.jface.bindings.keys.KeySequence;
import org.eclipse.jface.text.contentassist.ContentAssistant;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.keys.IBindingService;
import org.eclipse.ui.texteditor.ITextEditorActionDefinitionIds;

/**
 * Specialization of the actual frame that displays the completions proposals.
 * 
 * TODO: HTML presentation
 * TODO: Show correct key binding for cycling
 * 
 * Mostly copied from pydev.
 * 
 * @author shellcomputer
 */
public class ApCoCoAContentAssistant extends ContentAssistant {
	
	/**
     * Keeps a boolean indicating if the last request was an auto-activation or not.
     */
    private boolean lastAutoActivated;
    
    /**
     * The number of times this content assistant has been activated.
     */
    public int lastActivationCount;
	
	public  ApCoCoAContentAssistant() {
		this.enableAutoInsert(true);
        this.lastAutoActivated = true;
        
        try{
            setRepeatedInvocationMode(true);
        }catch(Exception e){
            //no need to log
        }
        
        try{
            setRepeatedInvocationTrigger(getCommandKeyBinding(ITextEditorActionDefinitionIds.CONTENT_ASSIST_PROPOSALS));
        }catch(Exception e){
            //no need to log
        }
        
        try{
            setStatusLineVisible(true);
        }catch(Exception e){
            //no need to log
        }
	}
	
	
	 
    /**
     * Shows the completions available and sets the lastAutoActivated flag
     * and updates the lastActivationCount.
     */
    @Override
    public String showPossibleCompletions() {
        lastActivationCount += 1;
        lastAutoActivated = false;
        try {
            return super.showPossibleCompletions();
        } catch (RuntimeException e) {
//            Throwable e1 = e;
//            while(e1.getCause() != null){
//                e1 = e1.getCause();
//            }
//            if(e1 instanceof JDTNotAvailableException){
//                return e1.getMessage();
//            }
            throw e;
        }
    }
    
    /**
     * @return true if the last tim was an auto activation (and updates
     * the internal flag regarding it).
     */
    public boolean getLastCompletionAutoActivated(){
        boolean r = lastAutoActivated;
        lastAutoActivated = true;
        return r;
    }

//    public void setIterationStatusMessage(String string) {
//        setStatusMessage(StringUtils.format(string, getIterationGesture()));
//    }
    
//    private String getIterationGesture() {
//        TriggerSequence binding = KeyBindingHelper.getContentAssistProposalBinding();
//        return binding != null ? binding.format(): "completion key";
//    }

    /**
     * Available for stopping the completion.
     */
    @Override
    public void hide(){
        super.hide();
    }
    
//    public static IInformationControlCreator createInformationControlCreator(ISourceViewer sourceViewer) {
//        return new IInformationControlCreator() {
//            public IInformationControl createInformationControl(Shell parent) {
//                return new DefaultInformationControl(parent, new PyInformationPresenter());
//            }
//        };
//    }
	
	   /**
     * @param commandId the command we want to know about
     * @return the 'best' key sequence that will activate the given command
     */
    public static KeySequence getCommandKeyBinding(String commandId) {
        final IBindingService bindingSvc = (IBindingService) PlatformUI.getWorkbench().getAdapter(IBindingService.class);
        TriggerSequence binding = bindingSvc.getBestActiveBindingFor(commandId);
        if (binding instanceof KeySequence){
            return (KeySequence) binding;
        }
        
        return null;
    }
}
