/**************************************************************************
 * This file is part of the source of ApCoCoA Eclipse Plugin Project 
 * (Moccha).
 *
 *  Copyright (c) 2009 Stefan Schuster
 *
 * Visit http://apcocoa.org/ for more information regarding ApCoCoA
 * and ApCoCoALib.
 * Visit http://www.apcocoa.org/wiki/ApCoCoA:KnownIssues for bugs, problems 
 * and known issues.
 *
 * Moccha is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3 or later)
 * as published by the Free Software Foundation. A copy of the full
 * licence may be found in the file COPYING in this directory.
 *
 * Moccha is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Moccha; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 **************************************************************************/
package org.apcocoa.eclipse.editors;

import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.rules.EndOfLineRule;
import org.eclipse.jface.text.rules.IPredicateRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.MultiLineRule;
import org.eclipse.jface.text.rules.RuleBasedPartitionScanner;
import org.eclipse.jface.text.rules.SingleLineRule;
import org.eclipse.jface.text.rules.Token;

public class ApCoCoAPartitionScanner extends RuleBasedPartitionScanner {

	public static final String APCOCOA_COMMENT = "__apcocoa_comment";
	
	public static final String APCOCOA_STRING = "__apcocoa_string";

	public ApCoCoAPartitionScanner() {
		IToken commentToken = new Token(APCOCOA_COMMENT);
		IToken defaultToken = new Token(IDocument.DEFAULT_CONTENT_TYPE);
		IToken stringToken = new Token(APCOCOA_STRING);
		IPredicateRule[] rules = new IPredicateRule[5];
		
		// rules[0] = new SingleLineRule("'", "'", defaultToken);
		// rules[1] = new SingleLineRule("\"", "\"", defaultToken);
		rules[0] = new MultiLineRule("\"", "\"", stringToken, '\\');
		rules[1] = new MultiLineRule("'", "'", stringToken);
		rules[2] = new MultiLineRule("/*", "*/", commentToken);
		rules[3] = new EndOfLineRule("//", commentToken);
		rules[4] = new EndOfLineRule("--", commentToken);

		setPredicateRules(rules);
	}

}
