/**************************************************************************
 * This file is part of the source of ApCoCoA Eclipse Plugin Project 
 * (Moccha).
 *
 *  Copyright (c) 2009 Stefan Schuster
 *
 * Visit http://apcocoa.org/ for more information regarding ApCoCoA
 * and ApCoCoALib.
 * Visit http://www.apcocoa.org/wiki/ApCoCoA:KnownIssues for bugs, problems 
 * and known issues.
 *
 * Moccha is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3 or later)
 * as published by the Free Software Foundation. A copy of the full
 * licence may be found in the file COPYING in this directory.
 *
 * Moccha is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Moccha; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 **************************************************************************/
package org.apcocoa.eclipse.editors;

import java.util.List;
import java.util.Vector;

/**
 * A simple datastructure for the ApCoCoA history.
 * Is currently not intended to be stored between sessions.
 * 
 * FIXME: Handle full history list
 * 
 * @author Stefan Schuster
 *
 */
public class CommandHistory {

	
	private List<String> fCommands;
	
	private final int fMaxLength;
	
	private int browserPos;
	
	public CommandHistory() {
		this(500);
	}
	
	public CommandHistory(int maxLength) {
		fMaxLength = maxLength;
		fCommands = new Vector<String>(fMaxLength);
		browserPos = 0;
	}
	
	public boolean reachedTop() {
		return browserPos == fCommands.size();
	}
	
	public boolean reachedBottom() {
		return browserPos<=0;
	}
	
	/**
	 * Reset the browsing-pointer, i.e. set it to the top element.
	 */
	public void reset() {
		browserPos = fCommands.size();
	}
	
	public void clear() {
		fCommands.clear();
		browserPos = 0;
	}
	
	public String forward() {
		String ret;
		if (browserPos < fCommands.size())
			browserPos++;
		if (browserPos == fCommands.size())
			return "";
		else 
			ret = fCommands.get(browserPos);
		return ret;
	}
	
	public String back() {
		if (browserPos > 0) browserPos--;
		if (browserPos >= 0 && fCommands.size() != 0) return fCommands.get(browserPos);
		else return "";
	}
	
	public void add(String s) {
		boolean reachedTopBefore = reachedTop(); 
		fCommands.add(s);
		if (reachedTopBefore) browserPos = fCommands.size();
	}
	
	public void enter(String s) {
		add(s);
		reset();
	}
	
	public void parseHistory(String s) {
		
	}
	
	public void getHistory() {
		
	}
	
}
