package org.apcocoa.eclipse.editors;

import java.io.File;
import java.io.FileFilter;
import java.io.FilenameFilter;

public abstract class RecursiveFileTraverser {

	private File fParentDir = null;
	
	private FilenameFilter fFilenameFilter = null;
	
	public RecursiveFileTraverser(File parentDir) {
		fParentDir = parentDir;
	}
	
	public RecursiveFileTraverser(File parentDir, FilenameFilter filter) {
		fParentDir = parentDir;
		fFilenameFilter = filter;
	}
	
	public void traverse() {
		if (!fParentDir.isDirectory())
			return;
		traverseDir(fParentDir);
	}
	
	private void traverseDir(File dir) {
		if (!dir.isDirectory())
			return;
		File[] files = null;
		if (fFilenameFilter == null) {
			files = dir.listFiles();
		} else {
			files = dir.listFiles(fFilenameFilter);
		}
		for (int i=0; i<files.length; i++) {
			if (files[i].isFile() && files[i].canRead())
				handleFile(files[i]);
		}
		File[] childDirs = dir.listFiles(new FileFilter() {
			public boolean accept(File pathname) {
				return pathname.isDirectory();
			}
		});
		for (int i=0; i<childDirs.length;i++)
			traverseDir(childDirs[i]);
	}
	
	protected abstract void handleFile(File file);
	
}
