/**************************************************************************
 * This file is part of the source of ApCoCoA Eclipse Plugin Project 
 * (Moccha).
 *
 *  Copyright (c) 2009 Stefan Schuster
 *
 * Visit http://apcocoa.org/ for more information regarding ApCoCoA
 * and ApCoCoALib.
 * Visit http://www.apcocoa.org/wiki/ApCoCoA:KnownIssues for bugs, problems 
 * and known issues.
 *
 * Moccha is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3 or later)
 * as published by the Free Software Foundation. A copy of the full
 * licence may be found in the file COPYING in this directory.
 *
 * Moccha is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Moccha; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 **************************************************************************/
package org.apcocoa.eclipse.preferences;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.preferences.AbstractPreferenceInitializer;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.SWT;

import org.apcocoa.eclipse.ApCoCoAPlugin;
import org.apcocoa.eclipse.IApCoCoAConstants;
import org.apcocoa.eclipse.IExternalContribution;

/**
 * Class used to initialize default preference values.
 */
public class PreferenceInitializer extends AbstractPreferenceInitializer {

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.runtime.preferences.AbstractPreferenceInitializer#initializeDefaultPreferences()
	 */
	public void initializeDefaultPreferences() {
		IPreferenceStore store = ApCoCoAPlugin.getDefault().getPreferenceStore();
		store.setDefault(IApCoCoAConstants.PREF_COMMANDS_COLOR, "10,10,10");
		store.setDefault(IApCoCoAConstants.PREF_RESPONSE_COLOR, "150,150,150");
		store.setDefault(IApCoCoAConstants.PREF_AC_SERVER_PORT, "");
		store.setDefault(IApCoCoAConstants.PREF_COCOA_AUTOSTART, true);
		store.setDefault(IApCoCoAConstants.PREF_CONSOLE_WIDTH, 80);
		store.setDefault(IApCoCoAConstants.PREF_USE_CONSOLE_WIDTH, false);
		doAcPathInitializing(store);
	}

	private void doAcPathInitializing(IPreferenceStore store) {
		
		IExternalContribution ec = ApCoCoAPlugin.getPlugin().getExternalContribution();
		
		if (ec == null) {
			ApCoCoAPlugin.debugLog("ec is null!");
			store.setDefault(IApCoCoAConstants.PREF_AC_EXEC, "");
			store.setDefault(IApCoCoAConstants.PREF_AC_WORDLIST_FILE_PATH, "");
			store.setDefault(IApCoCoAConstants.PREF_AC_SERVER_EXEC, "");
			store.setDefault(IApCoCoAConstants.PREF_AC_PACKAGES_PATH, "");
			store.setDefault(IApCoCoAConstants.PREF_AC_COCOARC_PATH, "");
		} else {
			ApCoCoAPlugin.debugLog("ec is NOT null!");
			String caTextPath = ec.getPathFor(IExternalContribution.PROPERTY_COCOA_TEXT_PATH);
			store.setDefault(IApCoCoAConstants.PREF_AC_EXEC, caTextPath);
			
			String wordlistPath = ec.getPathFor(IExternalContribution.PROPERTY_WORDLIST_PATH);
			store.setDefault(IApCoCoAConstants.PREF_AC_WORDLIST_FILE_PATH, wordlistPath);
			
			String caServerPath = ec.getPathFor(IExternalContribution.PROPERTY_APCOCOA_SERVER_PATH);
			store.setDefault(IApCoCoAConstants.PREF_AC_SERVER_EXEC, caServerPath);
			
			String caPackagesPath = ec.getPathFor(IExternalContribution.PROPERTY_PACKAGES_PATH);
			store.setDefault(IApCoCoAConstants.PREF_AC_PACKAGES_PATH, caPackagesPath);
			
			String caRcFilePath = ec.getPathFor(IExternalContribution.PROPERTY_COCOARC_PATH);
			store.setDefault(IApCoCoAConstants.PREF_AC_COCOARC_PATH, caRcFilePath);
		}
	}

	
//	private void doAcPathInitializing_(IPreferenceStore store) {
//		
//		IExtensionRegistry reg = Platform.getExtensionRegistry();
//		IConfigurationElement[] extensions = reg
//				.getConfigurationElementsFor("org.apcocoa.external.contribution");
//		
//		if (extensions.length == 0 || extensions.length > 1)
//			return; // TODO: throw exception
//		
//		IConfigurationElement element = extensions[0];
//		
//		String pluginLocation;
//		try {
//			IExternalContribution extContrib = (IExternalContribution) element.createExecutableExtension("class");
//			pluginLocation = String.valueOf(extContrib.getPluginLocation());
//			System.out.println("PrefInit: Got plug-in location :" + pluginLocation);
//		} catch (CoreException e) {
//			ApCoCoAPlugin.log("Could not access ApCoCoA contribution plugin to access preferences paths.", e);
//			return;
//		}
//		
//		String caTextPath = pluginLocation + element.getAttribute("cocoa_text_path");
//		store.setDefault(IApCoCoAConstants.PREF_AC_EXEC, caTextPath);
//		
//		String wordlistPath = pluginLocation + element.getAttribute("wordlist_path");
//		store.setDefault(IApCoCoAConstants.PREF_AC_WORDLIST_FILE_PATH, wordlistPath);
//		
//		String caServerPath = pluginLocation + element.getAttribute("cocoa_server_path");
//		store.setDefault(IApCoCoAConstants.PREF_AC_SERVER_EXEC, caServerPath);
//		
//		String caPackagesPath = pluginLocation + element.getAttribute("packages_path");
//		store.setDefault(IApCoCoAConstants.PREF_AC_PACKAGES_PATH, caPackagesPath);
//		
//		String caRcFilePath = pluginLocation + element.getAttribute("rc_file_path");
//		store.setDefault(IApCoCoAConstants.PREF_AC_COCOARC_PATH, caRcFilePath);
//		
//	}

}
