package org.apcocoa.eclipse.wizards;

import org.apcocoa.eclipse.ApCoCoAPlugin;
import org.eclipse.core.resources.IFile;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.dialogs.WizardNewFileCreationPage;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.internal.ide.DialogUtil;
import org.eclipse.ui.internal.wizards.newresource.ResourceMessages;
import org.eclipse.ui.wizards.newresource.BasicNewResourceWizard;

public abstract class AbstractApCoCoAFile extends Wizard implements INewWizard {

	private IStructuredSelection selection;
	private WizardNewFileCreationPage newFileWizardPage;
	private IWorkbench workbench;

	public AbstractApCoCoAFile(String title) {
		setWindowTitle(title);
	}
	
	@Override
	public void addPages() {

		newFileWizardPage = getCorrectWizardPage(selection);
		addPage(newFileWizardPage);
	}

	abstract protected WizardNewFileCreationPage getCorrectWizardPage(
			IStructuredSelection selection) ;
	
	@Override
	public boolean performFinish() {
		IFile file = newFileWizardPage.createNewFile();
		
		
		if (file == null)
			return false;
		
		IWorkbenchWindow dw = ApCoCoAPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow();
        BasicNewResourceWizard.selectAndReveal(file, dw);
		
		// Open editor on new file.
        try {
            if (dw != null) {
                IWorkbenchPage page = dw.getActivePage();
                if (page != null) {
                    IDE.openEditor(page, file, true);
                }
            }
        } catch (PartInitException e) {
            DialogUtil.openError(dw.getShell(), ResourceMessages.FileResource_errorMessage, 
                    e.getMessage(), e);
        }

        return true;
		
	}

	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.workbench = workbench;
		this.selection = selection;
	}
	
}
