package org.apcocoa.launch;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.apcocoa.eclipse.ApCoCoAPlugin;
import org.apcocoa.eclipse.IApCoCoAConstants;
import org.eclipse.cdt.utils.spawner.ProcessFactory;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.model.IProcess;
import org.eclipse.debug.core.model.LaunchConfigurationDelegate;


public class ApcocoaLaunchDelegate extends LaunchConfigurationDelegate {

	private ProcessTerminationCheck fTerminationCheck;
	
	public ApcocoaLaunchDelegate() {
		super();
	}
	
	public void launch(ILaunchConfiguration configuration, String mode,
			ILaunch launch, IProgressMonitor monitor) throws CoreException {
		
		// TODO We currently ignore the launch and set everything here. 
		
		Process spawner;
		
		String[] cocoaPath = new String[] {ApCoCoAPlugin.getPlugin().getPreferenceStore().getString(IApCoCoAConstants.PREF_AC_EXEC)};
		File cocoaPathAsFile = new File(cocoaPath[0]);
		if (! cocoaPathAsFile.exists())
			throw new CoreException(new Status(IStatus.ERROR, "ApcocoaLaunchDelegate", 
					"The given ApCoCoA executable does not exist. Please check the preferences."));
		
		File workingDir = (new File(cocoaPath[0])).getParentFile().getAbsoluteFile();
		
		String[] environment = {};
		
		try {

			ProcessFactory factory = ProcessFactory.getFactory();
			// if (PTY.isSupported()) {
			//	pty = new PTY(false);
			//	spawner = factory.exec(commandLine, environment, workingDirectoryAsFile, pty);
			//}
			//else {

				// spawner = factory.exec(commandLine, environment, workingDirectoryAsFile);
				spawner = factory.exec(cocoaPath, environment, workingDir);
		//	}
		}
		catch (IOException exception) {

			Status error;
			String message = exception.getMessage();
			error = new Status(IStatus.ERROR, "TODO ID", message, exception);
			throw new CoreException(error);
		}
		
		// Currently disabled.
		// fTerminationCheck = new ProcessTerminationCheck(spawner);
		// fTerminationCheck.start();
		
		Map processAttributes = new HashMap();
		String programName = "apcocoa_text";
		processAttributes.put(IProcess.ATTR_PROCESS_TYPE, programName.toLowerCase());
		// Create the IProcess:
		//
		IProcess process = null;
		if (spawner != null) {

			String[] configurationName = {configuration.getName()};
			process = DebugPlugin.newProcess(launch, spawner, cocoaPath[0], processAttributes);
		}		
	}

	public class ProcessTerminationCheck extends Thread {
		
		private Process fProcess;
		
		private boolean fHasTerminated;
		
		public ProcessTerminationCheck(Process p) {
			fProcess = p;
		}
		
		public void run(){
			while(!fHasTerminated) {
				try {
					synchronized (this) {
						this.wait(1000);
					}
					int s = fProcess.exitValue();
					System.out.println("Terminated!!!!!!!!!!!!!!!!");
					fHasTerminated = true;
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (IllegalThreadStateException e) {
					System.out.println("Running.........");
				}
			}
		}
	}
	
}
